@@ -1,7 +1,7 @@
-use std::ptr;
-use std::thread;
 use std::cell::UnsafeCell;
+use std::ptr;
 use std::sync::atomic::{AtomicPtr, Ordering};
+use std::thread;
 
 struct Node<T> {
     next: AtomicPtr<Node<T>>,
@@ -94,6 +94,12 @@ impl<T> Queue<T> {
     }
 }
 
+impl<T> Default for Queue<T> {
+    fn default() -> Self {
+        Queue::new()
+    }
+}
+
 impl<T> Drop for Queue<T> {
     fn drop(&mut self) {
         while let Some(_) = self.pop() {}
@@ -103,9 +109,9 @@ impl<T> Drop for Queue<T> {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use std::thread;
     use std::sync::mpsc::channel;
     use std::sync::Arc;
+    use std::thread;
 
     #[test]
     fn test_queue() {
